private import SDL;
private import opengl;

private import log;

class Texture {
	private GLuint id_;
	private GLfloat texcoord_[4];

    /* Quick utility function for texture creation */
	int power_of_two(int input) {
		int value = 1;

		while ( value < input ) {
			value <<= 1;
		}
		return value;
	}

	this(char[] filename) {
		Log.info(filename ~ ": loading texture");

		SDL_Surface* surf = SDL_LoadBMP(filename);
		this(surf);
		SDL_FreeSurface(surf);
	}

	this(SDL_Surface *surface) {
		GLuint texture;
		int w, h;
		SDL_Surface *image;
		SDL_Rect area;
		Uint32 saved_flags;
		Uint8  saved_alpha;

		/* Use the surface width and height expanded to powers of 2 */
		w = power_of_two(surface.w);
		h = power_of_two(surface.h);
		texcoord_[0] = 0.0f;			/* Min X */
		texcoord_[1] = 0.0f;			/* Min Y */
		texcoord_[2] = (GLfloat)surface.w / w;	/* Max X */
		texcoord_[3] = (GLfloat)surface.h / h;	/* Max Y */

		version(LittleEndian) {
			image = SDL_CreateRGBSurface(
				SDL_SWSURFACE,
				w, h,
				32,
				0x000000FF, 
				0x0000FF00, 
				0x00FF0000, 
				0xFF000000);
		}
		version(BigEndian) {
			image = SDL_CreateRGBSurface(
				SDL_SWSURFACE,
				w, h,
				32,
				0xFF000000,
				0x00FF0000, 
				0x0000FF00, 
				0x000000FF);
		}

		if ( image == null ) {
			Log.error("cannot load texture");
		}

		/* Save the alpha blending attributes */
		saved_flags = surface.flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
		saved_alpha = surface.format.alpha;
		if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
			SDL_SetAlpha(surface, 0, 0);
		}

		/* Copy the surface into the GL texture image */
		area.x = 0;
		area.y = 0;
		area.w = surface.w;
		area.h = surface.h;
		SDL_BlitSurface(surface, &area, image, &area);

		/* Restore the alpha blending attributes */
		if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
			SDL_SetAlpha(surface, saved_flags, saved_alpha);
		}

		/* Create an OpenGL texture for the image */
		glGenTextures(1, &texture);
		glBindTexture(GL_TEXTURE_2D, texture);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 w, h,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 image.pixels);
		SDL_FreeSurface(image); /* No longer needed */

		id_ = texture;
	}

	~this() {
		deleteTexture();
	}

	void bind() {
		glBindTexture(GL_TEXTURE_2D, id_);
	}

	GLfloat[] texcoord() {
		return texcoord_;
	}

	void deleteTexture() {
		glDeleteTextures(1, &id_);
	}

}

